
# q-expansion of newform 5586.2.a.d, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5586
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [-1], [-1], [0], [-5], [4], [-2], [-1], [6], [-3], [1], [6], [-6], [-4], [6], [-3], [-9], [-10], [10], [-16], [6], [7], [9], [8], [3], [10], [8], [-13], [-20], [-10], [-7], [13], [-10], [20], [-18], [-1], [-12], [4], [24], [-2], [0], [16], [12], [7], [10], [16], [8], [-21], [-3], [-20], [-8], [24], [-9], [-3], [18], [-16], [3], [11], [6], [30], [12], [-9], [10], [2], [-9], [9], [30], [31], [-32], [-20], [30], [-18], [25], [4], [8], [2], [6], [-28], [8], [31], [20], [30], [10], [14], [37], [23], [26], [-1], [26], [-32], [12], [-10], [13], [15], [-6], [22], [-21], [6], [-38], [32], [42], [9], [11], [16], [40], [-27], [-21], [-4], [32], [-1], [47], [26], [-32], [22], [-25], [-44], [40], [-24], [-15], [-8], [8], [19], [21], [-13], [-46], [-33], [4], [6], [-1], [-44], [-24], [46], [-7], [36], [28], [15], [-26], [-18], [1], [30], [28], [-33], [48], [-35], [28], [-34], [12], [-10], [-26], [4], [2], [42], [6], [6], [2], [-38], [-48], [28], [-7], [53], [36], [34], [5], [-15], [26], [52], [-37], [50], [-51], [-49], [-49], [-4], [32], [53], [-45], [-36], [-26], [1], [16], [-64], [-3], [21], [-54], [40], [-30], [-21], [38], [8], [49], [-4], [-51], [57], [8], [-2], [-21], [-8], [-62], [-14], [-22], [24], [-5], [20], [-16], [-6], [28], [-15], [-16], [-41], [58], [-52], [-6], [-57], [-23], [-3], [28], [-46], [-53], [-64], [26], [-10], [-6], [-56], [58], [41], [-39], [-50], [12], [46], [-27], [0], [-46], [52], [-33], [44], [-50], [-8], [1], [9], [28], [42], [-48], [68], [49], [-38], [-26], [42], [47], [12], [30], [36], [-8], [-50], [-68], [6], [-23], [-69], [-28], [-54], [-73], [-53], [-40], [-15], [4], [-70], [0], [12], [27], [14], [22], [1], [64], [-2], [-1], [-49], [-35], [52], [9], [46], [-22], [-4], [70], [13], [54], [-2], [-58], [48], [3], [-53], [77], [-30], [6], [-9], [22], [-4], [-36], [23], [40], [78], [4], [72], [-6], [51], [27], [13], [28], [2], [-35], [76], [-36], [-4], [14], [54], [-8], [80], [-42], [66], [29], [60], [-17], [-56], [-10], [22], [15], [20], [-46], [7], [80], [-36], [58], [57], [-38], [43], [80], [48], [-92], [-68], [56], [-6], [-53], [69], [-34], [72], [39], [45], [-3], [27], [20], [26], [6], [1], [-4], [85], [50], [19], [-32], [76], [-50], [93], [-78], [54], [-8], [78], [-24], [60], [-80], [37], [-46], [-56], [71], [81], [-20], [-54], [-15], [-4], [-40], [-92], [20], [21], [-40], [6], [-62], [-66], [16], [12], [52], [28], [-80], [8], [-32], [86], [79], [-42], [-81], [-12], [-96], [-7], [-53], [82], [-12], [57], [-20], [102], [88], [-28], [55], [25], [-52], [-2], [-84], [41], [-21], [13], [71], [-6], [-62], [-47], [-68], [-40], [-2], [64], [63], [-40], [18], [47], [-94], [-94], [3], [-24], [-28], [-58]]
